import json


class Destination():

    def __init__(self, connector, form_id):
        self.connector = connector
        self.form_id = form_id
        self.base_url = 'https://api.devicemagic.com/api/forms/' \
                        '{0}/destinations'.format(self.form_id)
        self.headers = {'Content-Type': 'application/json'}
        self.format = 'json'

    def all(self):
        path = self.base_url + '.' + self.format
        request = self.connector.execute_request(path, 'GET')
        return request

    def details(self, destination_id):
        path = self.base_url + '/' + str(destination_id) + '.' + self.format
        request = self.connector.execute_request(path, 'GET')
        return request

    def create(self, json, form_id=None):
        if form_id is not None:
            path = 'https://www.devicemagic.com/api/'\
                   'forms/{0}/destinations.json'.format(form_id)
        else:
            path = self.base_url + '.' + self.format

        request = self.connector.execute_request(
            path, 'POST', data=json, headers=self.headers)
        return request

    def update(self, destination_id, json):
        path = self.base_url + '/' + str(destination_id) + '.' + self.format
        request = self.connector.execute_request(
            path, 'PUT', data=json, headers=self.headers)
        return request

    def delete(self, destination_id):
        path = self.base_url + '/' + str(destination_id)
        print(path)
        request = self.connector.execute_request(path, 'DELETE')
        if request.status_code >= 200 and request.status_code < 300:
            return 'Destination deleted'
        else:
            return self.connector.failed_request_details(request)

    def copy(self, destination_id, form_id=None):
        destination_to_copy = self.details(destination_id)
        format = destination_to_copy['destination']['format_type']
        transport = destination_to_copy['destination']['transport_type']
        binary = destination_to_copy['destination']['binary_transport_type']
        destination_elements = [format, transport, binary]
        self._remove_keys(destination_elements, destination_to_copy)
        new_destination_json = self._format_destination_json(
            destination_to_copy, format,
            transport, binary)
        return self.create(new_destination_json, form_id=form_id)

    def _remove_keys(self, destination_elements, json):
        for element in destination_elements:
            if element is None:
                continue
            del json['destination'][element]['id']
            del json['destination'][element]['created_at']
            del json['destination'][element]['updated_at']

    def _format_destination_json(self, destination_to_copy,
                                 format, transport, binary):
        new_json = {}
        new_json['destination'] = {}
        new_json['destination']['description'] = \
            destination_to_copy['destination']['description']
        new_json['destination']['format_selection'] = format
        new_json['destination']['transport_selection'] = transport
        new_json['destination']['binary_transport_selection'] = binary
        new_json[format] = destination_to_copy['destination'][format]
        new_json[transport] = destination_to_copy['destination'][transport]
        if binary is not None:
            new_json[binary] = \
                destination_to_copy['destination'][binary]
        new_json = json.dumps(new_json)
        return new_json



class Group():

    def __init__(self, connector, org_id):
        self.connector = connector
        self.org_id = org_id
        self.base_url = 'https://api.devicemagic.com/api/v2/organizations/' \
                        '{0}/groups'.format(self.org_id)
        self.headers = {'Content-Type': 'application/json'}
        self.format = 'json'

    def all(self):
        path = self.base_url + '.' + self.format
        request = self.connector.execute_request(path, 'GET')
        return request

    def create(self, json):
        path = self.base_url
        request = self.connector.execute_request(
            path, 'POST', data=json, headers=self.headers, return_json=False)
        if request.status_code >= 200 and request.status_code < 300:
            return 'Group created'
        else:
            return self.connector.failed_request_details(request)

    def update(self, group_id, json):
        path = self.base_url + '/' + str(group_id)
        request = self.connector.execute_request(
            path, 'PUT', data=json, headers=self.headers, return_json=False)
        if request.status_code >= 200 and request.status_code < 300:
            return 'Group updated'
        else:
            return self.connector.failed_request_details(request)

    def delete(self, group_id):
        path = self.base_url + '/' + str(group_id)
        request = self.connector.execute_request(
            path, 'DELETE', return_json=False)
        if request.status_code >= 200 and request.status_code < 300:
            return 'Group deleted'
        else:
            return self.connector.failed_request_details(request)



class Database():

    def __init__(self, connector, id):
        self.connector = connector
        self.id = id
        self.base_url = 'https://api.devicemagic.com/api/forms' \
                        '/{0}/device_magic_database.json'.format(self.id)

    def json(self, *args):
        return self._filtered_query(args) if args else self._basic_query()

    def _basic_query(self):
        path = self.base_url
        request = self.connector.execute_request(path, 'GET')
        return request

    def _join_params(self, parameters):
        params = [param.strip() for param in parameters]
        params_for_url = '?' + '&'.join(params)
        return params_for_url

    def _filtered_query(self, parameters):
        params = self._join_params(parameters)
        path = self.base_url + params
        request = self.connector.execute_request(path, 'GET')
        return request



from __future__ import absolute_import
import os
import requests


class DeviceMagicAuth(requests.auth.AuthBase):

    def __init__(self, api_key=None):
        """
        Args:
            api_key (``str``): DeviceMagic API Key. Optional.
                If not set, it will look for
                enviroment variable ``DEVICEMAGIC_API_KEY``
        """
        try:
            self.api_key = api_key or os.environ['DEVICEMAGIC_API_KEY']
        except KeyError:
            raise KeyError('Api Key not found. Pass api_key as a kwarg \
                            or set an env var DEVICEMAGIC_API_KEY with your \
                            key')

    def __call__(self, request):
        auth_token = {'Authorization': '{0}'.format(self.api_key)}
        request.headers.update(auth_token)
        return request



from .auth import DeviceMagicAuth
import requests


class DeviceMagic():

    def __init__(self, args={}):
        self.api_key = args.get('api_key')
        self.session = requests.Session()
        self.session.auth = DeviceMagicAuth(self.api_key)
        self.form_id = str(args.get('form_id'))
        self.database_id = str(args.get('database_id'))
        self.org_id = str(args.get('org_id'))
        self.resource_id = str(args.get('resource_id'))
        self.file_path = args.get('file_path')
        self.device_identifier = args.get('device_identifier')

    @property
    def resource(self):
        from .resource import Resource
        return Resource(self.connector, self.file_path)

    @property
    def database(self):
        from .database import Database
        return Database(self.connector, self.database_id)

    @property
    def destination(self):
        from .destination import Destination
        return Destination(self.connector, self.form_id)

    @property
    def device(self):
        from .device import Device
        return Device(self.connector, self.org_id)

    @property
    def form(self):
        from .form import Form
        return Form(self.connector, self.org_id)

    @property
    def group(self):
        from .group import Group
        return Group(self.connector, self.org_id)

    @property
    def dispatch(self):
        from .dispatch import Dispatch
        return Dispatch(self.connector, self.org_id)

    @property
    def connector(self):
        from .connector import Connector
        return Connector(self.session)



class Connector(object):

    def __init__(self, session):
        self.r = session

    def execute_request(self, path, method,
                        data=None, headers={}, return_json=True):
        if method == 'GET':
            return self._get(path, return_json)
        elif method == 'POST':
            return self._post(path, data, headers, return_json)
        elif method == 'PUT':
            return self._put(path, data, headers, return_json)
        elif method == 'PATCH':
            return self._patch(path, data, headers)
        elif method == 'DELETE':
            return self._delete(path)
        else:
            raise ValueError('Invalid method provided: {}'.format(method))

    def _get(self, path, return_json=True):
        request = self.r.get(path)
        return request.json() if return_json else request.content

    def _post(self, path, data, headers, return_json=True):
        self.r.headers.update(headers)
        request = self.r.post(path, data=data, headers=self.r.headers)
        return request.json() if return_json else request

    def _put(self, path, data, headers, return_json=True):
        self.r.headers.update(headers)
        request = self.r.put(path, data=data, headers=self.r.headers)
        return request.json() if return_json else request

    def _patch(self, path, data, headers):
        self.r.headers.update(headers)
        request = self.r.patch(path, data=data, headers=self.r.headers)
        return request.json()

    def _delete(self, path):
        request = self.r.delete(path)
        return request

    def failed_request_details(self, request):
        return 'Failed with status code: {0} {1}, headers: {2}'.format(
            request.status_code, request.reason, request.headers)



class Dispatch():

    def __init__(self, connector, org_id):
        self.connector = connector
        self.org_id = org_id
        self.device_base_url = 'https://api.devicemagic.com/api/v2/devices'
        self.org_base_url = 'https://api.devicemagic.com/api/v2/organizations'
        self.headers = {'Content-Type': 'application/json'}
        self.format = 'json'

    def all(self, device_identifier=None):
        if device_identifier is not None:
            path = self.device_base_url + '/' + device_identifier \
                  + '/dispatches' + '.' + self.format
        else:
            path = self.org_base_url + '/' + self.org_id + '/dispatches' \
                + '.' + self.format
        request = self.connector.execute_request(path, 'GET')
        return request

    def push(self, device_identifier, json):
        path = self.device_base_url + '/' + device_identifier + '/dispatches' \
            + '.' + self.format
        request = self.connector.execute_request(
            path, 'POST', data=json, headers=self.headers)
        return request

    def update(self, device_identifier, dispatch_id, json):
        path = self.device_base_url + '/' + device_identifier + '/dispatches/'\
               + str(dispatch_id) + '.' + self.format
        request = self.connector.execute_request(
            path, 'PATCH', data=json, headers=self.headers)
        return request

    def delete(self, device_identifier, dispatch_id=None):
        if dispatch_id is not None:
            path = self.device_base_url + '/' + device_identifier \
                   + '/dispatches/' + str(dispatch_id) + '.' + self.format
            request = self.connector.execute_request(path, 'DELETE')
        else:
            path = self.device_base_url + '/' + device_identifier \
                   + '/dispatches/destroy_all' + '.' + self.format
            request = self.connector.execute_request(
                path, 'POST', headers=self.headers, return_json=False)
        if request.status_code >= 200 and request.status_code < 300:
            return 'Dispatch deleted'
        else:
            return self.connector.failed_request_details(request)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='pydevice',
    version='0.3.3',
    description="Device Magic API Wrapper",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/aseli1/pydevice',
    author='Anthony Seliga',
    author_email='anthony.seliga@gmail.com',
    license='MIT',
    packages=['pydevice'],
    python_requires='>3.0',
    install_requires=[
        'requests',
        'vcrpy',
        'pytest'
    ],
    zip_safe=False)



import base64
import mimetypes
import json


class Resource():

    def __init__(self, connector, file_path):
        self.connector = connector
        self.file_path = file_path
        self.base_url = 'https://api.devicemagic.com/api/resources'
        self.FORMAT = 'json'
        if self.file_path is not None:
            self.content_type = mimetypes.guess_type(self.file_path)
        else:
            self.content_type = None
        self.headers = {'Content-Type': 'application/json'}

    def all(self):
        path = self.base_url + '.' + self.FORMAT
        request = self.connector.execute_request(path, 'GET')
        return request

    def download(self, resource_id):
        path = self.base_url + '/' + str(resource_id) + '.' + self.FORMAT
        request = self.connector.execute_request(
            path, 'GET', return_json=False)
        return request

    def details(self, resource_id):
        path = self.base_url + '/' + str(resource_id) \
            + '/describe' + '.' + self.FORMAT
        request = self.connector.execute_request(path, 'GET')
        return request

    def _encode_file(self, file):
        return base64.b64encode(file).decode('utf-8')

    def _encode_local_file(self):
        with open(self.file_path, 'rb') as resource_file:
            self.encoded_file = self._encode_file(resource_file.read())

    def create(self, description, file_name,
               file_data=None, content_type=None):
        if file_data is None:
            self._encode_local_file()
        data = file_data or self.encoded_file
        content_type = content_type or self.content_type
        payload = {'resource': {'description': description,
                                'file': {'file_name': file_name,
                                         'file_data': data,
                                         'content_type': content_type}}}

        path = self.base_url + '.' + self.FORMAT
        request = self.connector.execute_request(
            path, 'POST', data=json.dumps(payload), headers=self.headers)
        return request

    def update(self, resource_id, description,
               file_name, file_data=None, content_type=None):
        if file_data is None:
            self._encode_local_file()
        data = file_data or self.encoded_file
        content_type = content_type or self.content_type
        payload = {'resource': {'description': description,
                                'file': {'file_name': file_name,
                                         'file_data': data,
                                         'content_type': content_type}}}
        path = self.base_url + '/' + str(resource_id) + '.' + self.FORMAT
        request = self.connector.execute_request(
            path, 'PUT', data=json.dumps(payload), headers=self.headers)
        return request

    def clone(self, resource_id, mimetype, description=None, file_name=None):
        file = self.download(resource_id)
        file_data = self._encode_file(file)
        description = description or 'CLONE - {0}'.format(
            self.details(resource_id)['resource']['description'])
        file_name = file_name or 'CLONE - {0}'.format(
            self.details(resource_id)['resource']['original_filename'])
        return self.create(description, file_name, file_data, mimetype)

    def delete(self, resource_id):
        path = self.base_url + '/' + str(resource_id)
        request = self.connector.execute_request(
            path, 'DELETE', return_json=False)
        if request.status_code >= 200 and request.status_code < 300:
            return 'Resource deleted'
        else:
            return self.connector.failed_request_details(request)
